:- module(rx, [ regcomp/2, regcomp/3,
		regexec/2, regexec/3, regexec/4,
		regfree/1,
		regmatch/2, regmatch/3,	regmatch/5
	      ]).

%% (c) G van Noord 1998
%% SICStus Prolog interface to ISO regular expression functions.

foreign(regcomp_pl,c,
	regcomp0(+chars,	      % pattern to be compiled
		 +integer,	      % reg_extended flag
		 +integer,	      % reg_newline flag
		 +integer,            % reg_icase flag
		 +integer,            % reg_nosub flag
		 [-address(regex_t)]  % returns address of compiled pattern
		)).

foreign(regexec_pl,c, 
	regexec0(+chars,              % string to be matched
		 +integer,            % reg_notbol flag
		 +integer,            % reg_noteol flag
		 +address(regex_t),   % address of compiled pattern
		 +integer,            % need return sub-strings?
		 [-term])).           % list of sub-strings (or [yes] or [])

foreign(regfree_pl,c,regfree(+address(regex_t))).

foreign_resource(rx,[regcomp_pl,regexec_pl,regfree_pl]).

:- load_foreign_resource(rx).

%% regcomp
%% flags: reg_extended reg_newline reg_icase reg_nosub
regcomp(String,CompiledPattern) :-
    regcomp0(String,0,0,0,0,CompiledPattern).

regcomp(String,CompiledPattern,Flags0) :-
    define_regcomp_flags(Flags0,0,F0,0,F1,0,F2,0,F3),
    regcomp0(String,F0,F1,F2,F3,CompiledPattern).

%% regexec
%% flags: reg_notbol reg_noteol 
regexec(String,CompiledPattern):-
    regexec0(String,0,0,CompiledPattern,0,[_|_]).

regexec(String,CompiledPattern,Result):-
    regexec0(String,0,0,CompiledPattern,1,Result).

regexec(String,CompiledPattern,Flags0,Result) :-
    define_regexec_flags(Flags0,0,F0,0,F1),
    regexec0(String,F0,F1,CompiledPattern,1,Result).

regmatch(Pattern,String) :-
    regcomp0(Pattern,0,0,0,1,CPattern),
    regexec0(String,0,0,CPattern,0,Val),
    regfree(CPattern),
    Val=[_|_].

regmatch(Pattern,String,Results):-
    regcomp0(Pattern,0,0,0,0,CPattern),
    regexec0(String,0,0,CPattern,1,Results0),
    regfree(CPattern),
    Results=Results0.

regmatch(Pattern,String,Results,CompilerFlags,ExecFlags):-
    regcomp(Pattern,CPattern,CompilerFlags),
    regexec(String,CPattern,ExecFlags,Results0),
    regfree(CPattern),
    Results=Results0.

define_regcomp(reg_extended, _, 1,F1,F1,F2,F2,F3,F3).
define_regcomp(reg_newline, F0,F0, _, 1,F2,F2,F3,F3).
define_regcomp(reg_icase,   F0,F0,F1,F1, _, 1,F3,F3).
define_regcomp(reg_nosub,   F0,F0,F1,F1,F2,F2, _, 1).

define_regexec(reg_notbol,_ , 1,F1,F1).
define_regexec(reg_noteol,F0,F0, _, 1).

define_regcomp_flags([],F0,F0,F1,F1,F2,F2,F3,F3).
define_regcomp_flags([H|T],F0a,F0,F1a,F1,F2a,F2,F3a,F3):-
    define_regcomp(H,F0a,F0b,F1a,F1b,F2a,F2b,F3a,F3b),
    define_regcomp_flags(T,F0b,F0,F1b,F1,F2b,F2,F3b,F3).

define_regexec_flags([],F0,F0,F1,F1).
define_regexec_flags([H|T],F0a,F0,F1a,F1):-
    define_regexec(H,F0a,F0b,F1a,F1b),
    define_regexec_flags(T,F0b,F0,F1b,F1).


